/**
 * Create one context for every application you plan to support.
 * Applications exist as a single Url and ALL processing/navigating is done through
 * this API.  The goal is to simplify web20 development while keeping some
 * important browser functionality, that would be lost without this library.
 * Both the reload and forward, back will work, including controling the title
 * that is displayed in the history.  We also implemented bean logic so that 
 * "beans" can be shared between javascript and servers, and there state can
 * be "replayed" on browser reloads and forward and back.
 * 
 * @param ourUrl - the relativeURL to where the application exists.
 * @param contexts - a comma seperated list of strings that are element ids (context ids)
 * @param beans -
 */
var w20 = { 
		contextUrl: "",	
		contexts: [],
		context: new Object,
		beans: [],
		bean: new Object,
		errorHandler: function() {},
		
		isOpera: function() { return (new RegExp('Opera').test(navigator.userAgent)); },
		isIE: function() { return (new RegExp('MSIE').test(navigator.userAgent)); },
		isIE6: function() {	return (new RegExp('MSIE 6').test(navigator.userAgent)); },
		isIE7: function() {	return (new RegExp('MSIE 7').test(navigator.userAgent)); },
		isSafari: function() { return (new RegExp('Safari').test(navigator.userAgent));	},
		isFirefox: function() { return (new RegExp('Firefox').test(navigator.userAgent)); },
		isFirefox2: function() { return (new RegExp('Firefox[\/\s]2').test(navigator.userAgent)); },
		isFirefox3: function() { return (new RegExp('Firefox[\/\s]3').test(navigator.userAgent)); },

		_addBean: function(s_bean) {
			eval('this.bean.' + s_bean + ' = new_' + s_bean + '();');
			eval('this.beans.push(this.bean.' + s_bean  + ');');
		},


		_addContext: function(s_context) {
			eval('this.context.' + s_context + ' = new_' + s_context + '();');
			eval('this.contexts.push(this.context.' + s_context  + ');');

		},

		/**
		 * This function must be called to initialize all contexts
		 * and beans
		 */
		initialize: function(ourUrl,lcontexts,lbeans) {

			if (lcontexts.length > 0) {
				var a_Contexts = lcontexts.split(",");

				for (var i = 0; i < a_Contexts.length;i++) {
					var s_context = a_Contexts[i];
					this._addContext(s_context);
				}
			}

			if (lbeans.length > 0) {
				var a_Beans = lbeans.split(",");

				for (var i = 0; i < a_Beans.length;i++) {
					var s_bean = a_Beans[i];
					this._addBean(s_bean);
				}
			}
		},



		replaceElementAsync: function(elementId, url, callbackFunc,indicatorStartFunc,indicatorFinishFunc)
		{	
			var _e;
			if (indicatorStartFunc) {
				try { indicatorStartFunc(); } catch (e) {_e = e;}
			}

			// Ready States
			var UNINITIALIZED = 0;
			var OPEN = 1;
			var SENT = 2;
			var RECEIVING = 3;
			var LOADED = 4;

			var req;
			try {
				req = new XMLHttpRequest();
			}
			catch (e) {
				try {
					req = new ActiveXObject("Msxml2.XMLHTTP");
				} catch (e) {
					req = new ActiveXObject("Microsoft.XMLHTTP");
				}
			}


			this.onReadyStateChange = function() {
				var _e;

				if (req.readyState == LOADED)
				{
					var element = document.getElementById(elementId);
					if (element == undefined) 
						alert(elementId + ' is not a valid id');
					if (element.tagName == 'TEXTAREA')
						document.getElementById(elementId).value = req.responseText;
					else
						document.getElementById(elementId).innerHTML = req.responseText;

					

					w20.activateJS(elementId);

					if (indicatorFinishFunc) {
						try { indicatorFinishFunc(); } catch(e) { _e = e; }
					}	

					if (callbackFunc) {
						try {
							callbackFunc();
						} catch(e) {
							w20.handleError("A " + e.name + " exception occured during callbackFunc:" + e.message);
						}
					}

				}

				if (_e != undefined)
					throw _e;
			}

			if (req == null) {
				w20.hadleError("Your browser does not support w20");
				return;
			}

			req.onreadystatechange = this.onReadyStateChange;
			req.open("GET", url, true);
			req.send(null);

			if (_e != undefined)
				throw _e;
		},


		/**
		 * 
		 * FIXME: test all variable types with all supported elements
		 */
		serializeElement: function(elementId) {
			var element = document.getElementById(elementId);
			if (element == undefined) {
				return undefined;
			}
			return element.value;
		},


		/**
		 * used to display a popup div over the existing page.  This div will block the 
		 * content behind it both visually and from UI interactions.  The developer should 
		 * position and size the id accordingly using css prior to this call,  I think 
		 * that when called this function will popup the div immediately and display a 
		 * loading message inside of it, including the closable icon.  This way if there is 
		 * an error loading the contents of the popup, the client can easily close it and 
		 * reattempt or avoid the popup.
		 */
		displayBlockingPopup: function(className,queryString,id, historyText) {	   
		},

		/**
		 * used to display a popup div over the existing page.  The div will block allow 
		 * the user to see and interact with the background.  The developer should position 
		 * and size the id accordingly using css prior to this call,  I think that when 
		 * called this function will popup the div immediately and display a loading message 
		 * inside of it, including the closable icon.  This way if there is an error loading 
		 * the contents of the popup, the client can easily close it and reattempt or avoid 
		 * the popup.
		 */
		displayNonBlockingPopup: function(className,queryString,id,historyText) {

		},

		/**
		 * Setting the title and adding history are combined for a reason.
		 * Typically you would design a document with the title at the top,
		 * followed by the body.  But because we need the title to be the 
		 * previous pages title at the time we take the bean snapshot and perform
		 * the addToHistory magic, we do it backwords inside of Application's.
		 * Only after all bean's are set to the correct value should you 
		 * attempt to call setTitle.
		 */
		setTitle: function(title) {
			document.title=title;
		},
		

		/**
		 * Add a spot in the clients browser history.  This function
		 * performs the magic of the history management
		 */
		addToHistory: function() {
			var state = w20.serialize();

			// Save Point perform all state maintainence
			w20.savePoint(state);
			
			if (w20.isIE()) {
				w20.monitorIframe();
			} else {			
				w20.monitorUrl();
			}
		},

		

		// Turns on monitoring of iframe 'historyManager' changes.
		//
		// onchange is a function that will be called.
		// onchange is of the form:
		// function onchange(newContent, oldContent);
		//
		// multiple calls will not cancel previous calls.
		monitorIframe: function()
		{
			manager = document.getElementById('historyManager');
			
			// If this is first == undefined, then this is either
			// an initial visit or reload.  Check the value of the
			// cookieStateHistory and if it's defined then this is 
			// a reload.  Replay the iframes to restore the lost browser
			// history.
			if(!manager) {
				// We define the iframe in html for maximum compatiability with Firefox
//				alert("w20 is not loaded properly (no iframe)");
				var ifram = document.createElement("iframe");
				ifram.style.height=0;
				ifram.style.width=0;
				ifram.style.border=0;
				ifram.id = "historyManager";

				ifram.src = "/servletForwarder?W20Title=" + escape(document.title) + "&W20IFramePos=" + w20.currentHistoryPos;
	
				w20.updatingIframe = true;
				w20.oldPos = w20.currentHistoryPos;
				manager = document.body.appendChild(ifram);
			} else {
				w20.updatingIframe = true;
				w20.oldPos = w20.currentHistoryPos;
				manager.src = "/servletForwarder?W20Title=" + escape(document.title) + "&W20IFramePos=" + w20.currentHistoryPos;
			}


			if(!monitorIframe.isOn) {

				monitorIframe.isOn = true;

				function checkIframe()
				{

					var manager = document.getElementById("historyManager");
					if (manager == undefined) {
						return;
					}

					var iframePosElement = w20.getContentDocument(manager).document.getElementById('W20IFramePos');
					if (iframePosElement == undefined) {
						return;
					}

					newPos = iframePosElement.value;
					
					var oldPos = w20.oldPos;
					if (oldPos == undefined) {
						w20.oldPos = oldPos = newPos;
					}

					

					if(w20.updatingIframe == undefined && oldPos != newPos) {		
						w20.oldPos = newPos;
						w20.loadFromHistory(newPos);
					}
				}

				setInterval(checkIframe, 45);
			}
		},
		
		// Compatibility function
		getContentDocument: function(ifram) {
			var bDoc;
			if (ifram.contentWindow != undefined) {
				bDoc = ifram.contentWindow;
				
			} else if (ifram.contentDocument != undefined) {
				bDoc = ifram.contentDocument;
			} else {
				bDoc = ifram;
			}
			
			return bDoc;
		},

		// This is called internally to perform all of the 
		// saving maintenence.
		savePoint: function(state)
		{
			
			// the lowHistoryPos will be used for treating the history like a fifo stack
			// limiting the number of states store in memory
			//w20.lowHistoryPos = w20.lowHistoryPos || 0;
			w20.states = w20.states || [];
			w20.currentHref = w20.currentHref || window. 	href;

			
			// We think starting with page 1 vs 0 looks more professional in the URL(#Page-0)
			if (w20.highHistoryPos == undefined) {
			    w20.highHistoryPos = 1;
			} else {
				w20.highHistoryPos++;
			}
			
			w20.states[w20.highHistoryPos] = state;
			w20.currentHistoryPos = w20.highHistoryPos;
			// this will us to keep history via replay on page reload
			//w20.updateHistoryCookie(w20.states);
			
		},
		
		/**
		 * Load the top N Entries from the HistoryCookie
		 * returns an associative array containing the
		 * state at each corresponding history element
		loadHistoryCookie: function() {
			return true;
		},
		
		updateHistoryCookie: function(states) {
			// serliaze all states (1..w20.highHistoryPos)
			// states are serialized like title:bean1Name:'bean1State':bean2Name:'bean2State':bean3Name:'bean3State'
			// I believe we can get away with just using a newline as a seperator between states.
			// So if w20.highHistoryPos is 3, then there will be 3 states stored on three lines like:

//title:bean1Name:'bean1State':bean2Name:'bean2State':bean3Name:'bean3State'
//title:bean1Name:'bean1State':bean2Name:'bean2State':bean3Name:'bean3State'
//title:bean1Name:'bean1State':bean2Name:'bean2State':bean3Name:'bean3State'


			var historyState = "";
			for (var i = 1;i <= w20.highHistoryPos;i++) {
				if (historyState != "")
					historyState = historyState + "\n";
				historyState = historyState + states[i - 1];
			}
			
		},
		 */
		

		loadFromHistory: function(pos) {
			w20.currentHistoryPos = pos;
//			alert("DETECTED CHANGE TO " + =);
			if (w20.states[pos] == undefined) {
				alert("We are sorry but your browser history was lost when you reloaded the page");
				return;
			}
			w20.deserialize(w20.states[pos]);
			
			// Any contexts that are reloaded within onHistoryChange will be have 
			// the request parameter GSOnHistoryChange defined.  This will be used
			// to prevent history based reloads from triggering setTitle logic.
			w20.isHistoryChange = true;
			if (w20.onHistoryChange != undefined) {
				try {
					w20.onHistoryChange();
				} catch (e) {
					w20.handleError("A " + e.name + " exception occured during onHistoryChange: " + e.message);
				}
			}
			w20.isHistoryChange = false;

		},
		isOn: false,
		//Turns on monitoring of url changes.
		//
		//onchange is a function that will be called.
		//onchange is of the form:
		//function onchange(newUrl, oldUrl);
		//
		//multiple calls will not cancel previous calls.
		monitorUrl: function()
		{
			w20.currentHref = '#Page-' + (w20.highHistoryPos);
			document.location.hash='Page-' + (w20.highHistoryPos);
			
			if(!w20.isOn) {
				w20.isOn = true;
				function checkUrl()
				{
					var newUrl = window.location.href;
					var oldUrl = w20.currentHref;
					
					
					w20.currentHref = newUrl;
					if(oldUrl != newUrl) {
						var iAnchor = newUrl.indexOf('#Page-');
						
						// if there is a # that is not the first char or
						// the newUrl's anchor is specified and is not oldUrl
						if (iAnchor <= 0 || newUrl.substr(iAnchor) != oldUrl) {
							// url changed
							var iOldAnchor = oldUrl.indexOf('#Page-');

							/**
							 * At one point this was useful
							if (iOldAnchor != -1) {
								var oldPos = parseInt( oldUrl.substr(iOldAnchor + '#Page-'.length) );
								if (oldPos == w20.highHistoryPos) {									
									// back was hit on the current document. we must add current state
									// to history
									w20.states[oldPos] = w20.serialize();
								}
								
							}
							 */
							
							var pos = newUrl.substr(iAnchor + "#Page-".length);

							if(pos < 0)
								return;
							
							w20.loadFromHistory(pos)

						}
					}
				}
				
				setInterval(checkUrl, 45);
			}
		},
		
		/**
		 * Add an action to the queue.  The queue will be replayed on page reload
		 * and/or history changes.
		 */
		addAsyncAction: function(type,className,queryString,id) {  
		},

		/**
		 * Remove an async action from the queue with the given element id.
		 * this queue any element under the given element will also be removed  
		 */
		removeAsyncAction: function(id) {
		},

		/**
		 * We provide support for persistant bean objects which contain only 
		 * getters and setters.  There constructor will first attempt to load
		 * from the request, and then fall back to this cookie.  This allows
		 * us to easily share these object between javascript and java with
		 * minimal effort. 
		 */
		saveParamsToCookie: function(cookieName, object) {
			var queryString="";
			for (var key in object) {
				value = object[key];
				if (queryString.length > 0) {
					queryString += "&";
				}
				queryString += key + "=" + escape(value);
			}
			var expires_date = new Date( today.getTime() + (1000 * 60 * 60 * 24 * 128));

			document.cookie = cookieName + "=" + queryString + ";expires=" + expires_date.toString();
		},

		/**
		 * Looks up the specified cookie and loads them into blankObject only if 
		 * the members exists in the object.
		 *
		 */
		getParamsFromCookie: function(cookieName, blankObject) {
			var query = _getCookie(cookieName);

			var vars = query.split("&");
			for (var i=0;i<vars.length;i++) {
				var pair = vars[i].split("=");

				if (blankObject[pair[0]] != undefined) {
					blankObject[pair[0]] = unescape(pair[1]);
				}
			} 

		},

		/**
		 * http://techpatterns.com/downloads/javascript_cookies.php
		 */
		_getCookie: function( check_name ) {
			// first we'll split this cookie up into name/value pairs
			// note: document.cookie only returns name=value, not the other components
			var a_all_cookies = document.cookie.split( ';' );
			var a_temp_cookie = '';
			var cookie_name = '';
			var cookie_value = '';
			var b_cookie_found = false; // set boolean t/f default f

			for ( i = 0; i < a_all_cookies.length; i++ )
			{
				// now we'll split apart each name=value pair
				a_temp_cookie = a_all_cookies[i].split( '=' );


				// and trim left/right whitespace while we're at it
				cookie_name = a_temp_cookie[0].replace(/^\s+|\s+$/g, '');

				// if the extracted name matches passed check_name
				if ( cookie_name == check_name )
				{
					b_cookie_found = true;
					// we need to handle case where cookie has no value but exists (no = sign, that is):
					if ( a_temp_cookie.length > 1 )
					{
						cookie_value = unescape( a_temp_cookie[1].replace(/^\s+|\s+$/g, '') );
					}
					// note that in cases where cookie is initialized but no value, null is returned
					return cookie_value;
					break;
				}
				a_temp_cookie = null;
				cookie_name = '';
			}
			if ( !b_cookie_found )
			{
				return null;
			}
		},



		/**
		 * The function serializes the current state.  Currently each bean is serialized, and appended
		 * to a single string which we then return.  This string is intended to be passed to 
		 * deserialize(state).  Which will re-establish all beans to their serialized state.
		 * states are serialized like title:bean1Name:'bean1State':bean2Name:'bean2State':bean3Name:'bean3State'
		 */
		serialize: function() {
// For now I'm not serializing the title because it causes the ordering of jsSetTitle and jsAddToHistory to matter
// It should get reset when the contexts reload as part of what the user declare in jsOnHistoryChange			
			var state = escape(document.title);
//			var state = "";
			for ( i = 0; i < this.beans.length; i++ ) {			
				if (state.length > 0)
					state +=":";
				state += this.beans[i].beanName + ':\'' + this.beans[i].serialize() + '\'' ;
			}

			return state;
		},

		/**
		 * This function takes the given state and deserializes it
		 * to this w20 object
		 */
		deserialize: function(state) {
			var parts = state.split(':');
			var c = 0;
// For now I'm not serializing the title because it causes the ordering of jsSetTitle and jsAddToHistory to matter
// It should get reset when the contexts reload as part of what the user declare in jsOnHistoryChange			
			document.title = unescape(parts[c++]);
			
			for (; c < (parts.length - 2);c = c + 2) {
				var beanName = parts[c];
				var origBeanValue = parts[c + 1];

				/* strip the enclosing single quotes from the value */
				var beanValue = origBeanValue.substr(1,origBeanValue.length - 2);

				if (beanName.indexOf("'") != -1) {
					alert('deserialization mis-count (uncomment alert below to debug)');
					return;
				}
//				alert(c + ' - ' + beanName + ' - ' + origBeanValue);
				eval('if (this.bean.'+ beanName +' == undefined) { alert("' + beanName + ' is not a bean"); }');
				eval('this.bean.' + beanName + '.deserialize(beanValue);');
			}
			
			

		},

		/**
		 * Set the element with the given id to be display:block.  This is handy for 
		 * performing popups and such that do not
		 */
		showElement: function(id) {  
		},

		/**
		 * Set the element with the given id to be display:none
		 */
		hideElement: function(id) {  
		},

		activateJS: function(elementID)	{
			element = document.getElementById(elementID);
			head = document.getElementsByTagName("head")[0];
			scripts = element.getElementsByTagName("script");
			for (var i=0;i<scripts.length;i++) {
				var script = scripts[i];

				newScript = document.createElement("script");
				newScript.type = script.type;
				newScript.langauge = script.language;
				newScript.onreadystatechange=newScript.onerror=newScript.onload=w20.emptyFunc;
				newScript.text=script.innerHTML;  
				head.appendChild(newScript);


			}
		},



		emptyFunc: function() {}


		
		
}
