package com.getsnappy.snappyj;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.URLDecoder;
import java.text.ParseException;
import java.util.Hashtable;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.apple.eawt.ApplicationBeanInfo;
import com.getsnappy.util.ReflectionCache.ClassCache;
import com.getsnappy.util.ReflectionCache.ConstructorCache;

/**
 * Context's are html element which will be given web 2.0 loading/reloading capabilites.
 * Our library is designed to preserve state so that forward/back browser buttons work
 * as well as page reloads.<br>
 * 
 * StaticContext can be used for most cases.  More complex application may desire the
 * functionality of DynamicContexts
 * 
 * @author brian
 * @see StaticContext
 * @see DynamicContext
 *
 */
public abstract class Context {

   public abstract String getContextId();
   public abstract String getClassName();
   
   
   public Hashtable<String,Context> subContexts = new Hashtable<String,Context>();
   
   
   protected abstract JavaScriptLoadingIndicator getJavaScriptLoadingIndicator();
//   public abstract String getQueryString();
//   public abstract String 
//   protected CategoryChooserBean dynamicContextBean;
   
   
   
   /**
    * outputs content for the ApplicationInstance or HttpServlet defined by getClassName()
    * This function basically loads and calls the corresponding method in the
    * 
    * @param out
    * @param request
    * @param response
    * @param out
    * 
    * @throws IOException In the case of an IOException trying to print an appropriate error message.
    */   
   protected void output(Writer out, HttpServletRequest request, HttpServletResponse response) throws IOException , ServletException
   {
      out.flush();
      ServletForwarder forwarder = new ServletForwarder(getClassName());
      forwarder.doGet(request, response);      
   }

   /**
    *  This method will output a empty placeholder div of the specified width and height,
    *  followed by issueing a jsReload inside a JavaScript block, causing the contents of this
    *  context to be loaded asynchronously
    *  
    * @param out
    * @param request
    * @param response
    * @param width
    * @param height
    * @throws IOException
    * @throws ServletException
    */
   public void include(Writer out, HttpServletRequest request, HttpServletResponse response) throws IOException , ServletException
   {
      out.write("<div id=\"" + getContextId() + "\">\n");
      output(out,request,response);
      out.write("</div>\n");
   }
   
 
   
   /**
    *  This method will output a empty placeholder div of the specified width and height,
    *  followed by issueing a jsReload inside a JavaScript block, causing the contents of this
    *  context to be loaded asynchronously
    *  
    * @param out
    * @param request
    * @param response
    * @param width
    * @param height
    * @throws IOException
    * @throws ServletException
    */
   public void includeAsync(Writer out, HttpServletRequest request, HttpServletResponse response, int width, int height) throws IOException , ServletException
   {
      out.write("<div id=\"" + getContextId() + "\">\n");
      out.write("<div id=\"" + getContextId() + "Placeholder\" style=\"width: " + width + "px; height: " + height + "px;\">&nbsp;</div>");
      out.write("</div>\n");
      out.write("<script type=\"text/javascript\">\n");
      
      // When including a subrequest asynchronously we must pass along the
      // W20IsHistoryChange
      boolean isHistoryChange = request.getParameter("W20IsHistoryChange") != null;
      
      if (isHistoryChange)
         out.write("w20.isHistoryChange = true;\n");
      out.write(jsReload() + "\n");
      if (isHistoryChange)
         out.write("w20.isHistoryChange = false;\n");

      out.write("</script>\n");
   }

   
   /**
    * outputs content for the ApplicationInstance or HttpServlet defined by getClassName()
    * This function basically loads and calls the corresponding method in the
    * 
    * @param out
    * @param request
    * @param response
    * @throws IOException In the case of an IOException trying to print an appropriate error message.
    */   
   /*
   public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException , ServletException
   {
      response.flushBuffer();
      String className = getClassName();

      ServletForwarder forwarder = new ServletForwarder();
      forwarder.doPost(request, response);      
   }
   */
   
    
   
   /**
    * Generate a JavaScript function that returns a JavaScript object representing this context. 
    * This must be declared prior to creating the Web20Environment.
    * 
    * NOTE: like an other js function, this one outputs straight JavaScript and should be 
    * nested inside a <script type="text/javascript"> tag or JavaScript subrequest, or any other 
    * place straight JavaScript might be applicable.
    * 
    * @param out is a java.io.Writer we output to
    * @param request TODO
    * @throws IOException when we have problems writing to out
    */
   public void jsDeclareObject(Writer out) throws IOException {
      out.write("    contextId: '" + getContextId() + "',\n");
      out.write("    reload: function () {\n");
      out.write("      var extraParams = (w20.isHistoryChange ? '&W20IsHistoryChange=1' : '');\n");
      out.write("      w20.replaceElementAsync('" + getContextId() + "','" + Options.getServletForwarderUrl() + "?applicationClassName=" + getClassName() + "' + extraParams, undefined, this._indicatorStart,this.indicatorFinish);\n");
      out.write("    },\n");
      out.write("    _indicatorStart: ");
      getJavaScriptLoadingIndicator().jsStart(out);
      out.write(",\n");
      out.write("    _indicatorFinish: ");
      getJavaScriptLoadingIndicator().jsFinish(out);
      out.write(",\n");

   }
   
   /**
    * This method returns inline JavaScript that will reload the context.  All beans
    * should be persisted before calling this function.  If historyText is null
    * then your state is not saved for forward and back button functionallity.  If
    * queryString is equal to "" or null then it is not passed.
    * 
    * @return
    */
   public String jsReload() {
      return "w20.context." + getContextId() + ".reload();";
   }
   
   
   
   
   
   /**
    * This attaches the subContext to this context.
    * 
    * @param subContext
    * @return
    */
   /*
    * postponing until v1.1
   public String jsAttachContext(Context subContext) {
      subContexts.put(subContext.getContextId(), subContext));
   }
   */
}
